/****************************************************\
|*Land Mines                                        *|
|*Created By: Sam Tsuki(Aka Sam X HearTLesS)        *|
|*                                                  *|
|* Have Fun,                                        *|
|*           Sam Tsuki                              *|
\****************************************************/

/*
Description:
This plugin creates Land Mines and Warning Signs through map cfg files.

CMDs:
say /minesmenu		// Shows Land Mines menu

CVARs:
mines_enabled 1		// Enables Land Mines explosion
mines_visible 1		// Enables Land Mines t obe visible when spawned
mines_radius 200	// Maximum radius for Mines damage
mines_maxdamage 50	// Maximum damage a Mine can deal
mines_respawntime 20	// Seconds to respawn a Mine after explosion (Put to 0 to never respawn)
*/

#include <amxmodx>
#include <amxmisc>
#include <engine>
#include <fakemeta>

#define PLUGIN "Land Mines"
#define VERSION "1.0"
#define AUTHOR "Sam Tsuki"

new mines_enabled, mines_visible, mines_radius, mines_maxdamage, mines_respawntime
new boom, MenuID[33]
//--------------------------------------------------------------------------------------------------
public plugin_init()
{
	register_plugin(PLUGIN, VERSION, AUTHOR)
	
	mines_enabled = register_cvar("mines_enabled", "1")
	mines_visible = register_cvar("mines_visible", "1")
	mines_radius = register_cvar("mines_radius", "200")
	mines_maxdamage = register_cvar("mines_maxdamage", "50")
	mines_respawntime = register_cvar("mines_respawntime", "20")
	
	register_clcmd("say /minesmenu", "mines_menu", ADMIN_KICK, "- Shows Land Mines menu")
	
	register_clcmd("say_team /minesmenu", "mines_menu", ADMIN_KICK, "- Shows Land Mines menu")
	
	register_menucmd(register_menuid("[Land Mines menu]"),1023,"action_menu")
	
	register_touch("player", "land_mine", "mine_touched")
	
	set_task(0.1, "readminescfg")
	set_task(0.1, "readsignscfg")
}
//--------------------------------------------------------------------------------------------------
public plugin_precache()
{
	precache_model("models/w_adrenaline.mdl")
	precache_model("models/land_mines/warning_ns.mdl")
	precache_model("models/land_mines/warning_ew.mdl")
	boom = precache_model("sprites/zerogxplode.spr")
}
//--------------------------------------------------------------------------------------------------
public readminescfg()
{
	new configsdir[128]
	get_configsdir(configsdir, 127)
	
	new file[256], mapname[51]
	get_mapname(mapname, 50)
	format(file, 255, "%s/land_mines/%s_mines.cfg", configsdir, mapname)
	
	if(file_exists(file)) {
		new readdata[128], txtlen
		new parsedx[8], parsedy[8], parsedz[8]
		new fsize = file_size(file, 1)
		for(new line = 0; line < fsize - 1; ++line) {
			read_file(file, line, readdata, 127, txtlen)
			parse(readdata, parsedx, 7, parsedy, 7, parsedz, 7)
			mine_spawn(parsedx, parsedy, parsedz)
		}
	}
}
//--------------------------------------------------------------------------------------------------
public readsignscfg()
{
	new configsdir[128]
	get_configsdir(configsdir, 127)
	
	new file[256], mapname[51]
	get_mapname(mapname, 50)
	format(file, 255, "%s/land_mines/%s_signs.cfg", configsdir, mapname)
	
	if(file_exists(file)) {
		new readdata[128], txtlen
		new parsedx[8], parsedy[8], parsedz[8], parseddir[8]
		new fsize = file_size(file, 1)
		for(new line = 0; line < fsize - 1; ++line) {
			read_file(file, line, readdata, 127, txtlen)
			parse(readdata, parsedx, 7, parsedy, 7, parsedz, 7, parseddir, 7)
			switch(str_to_num(parseddir)) {
				case 0: sign_spawn_ns(parsedx, parsedy, parsedz)
				case 1: sign_spawn_ew(parsedx, parsedy, parsedz)
			}
		}
	}
}
//--------------------------------------------------------------------------------------------------
public mines_menu(id, level, cid)
{
	if(!cmd_access(id, level, cid, 0))
		return PLUGIN_HANDLED
	
	MenuID[id] = 1
	show_minesmenu(id)
	
	return PLUGIN_HANDLED
}
//--------------------------------------------------------------------------------------------------
public show_minesmenu(id)
{
	new menu_body[320]
	new n = 0
	new len = 319
	new keys
	
	switch(MenuID[id]) {
		case 1: {
			n += format( menu_body[n],len-n,"\yLand Mines menu:\w^n^n")
			
			switch(get_pcvar_num(mines_enabled)) {
				case 0: n += format( menu_body[n],len-n,"\y1.\w Turn Land Mines ON^n")
				case 1: n += format( menu_body[n],len-n,"\y1.\w Turn Land Mines OFF^n")
			}
			switch(get_pcvar_num(mines_visible)) {
				case 0: n += format( menu_body[n],len-n,"\y2.\w Turn Mines visiblity ON^n")
				case 1: n += format( menu_body[n],len-n,"\y2.\w Turn Mines visiblity OFF^n")
			}
			n += format( menu_body[n],len-n,"\y3.\w Edit Mode^n")
			
			n += format( menu_body[n],len-n,"^n\y0. Exit")
			
			keys = (1<<0)|(1<<1)|(1<<2)|(1<<9)
		}
		case 2: {
			n += format( menu_body[n],len-n,"\yLand Mines edit menu:\w^n^n")
			
			n += format( menu_body[n],len-n,"\y1.\w Create Mine^n")
			n += format( menu_body[n],len-n,"\y2.\w Create Warning sign (North/South)^n")
			n += format( menu_body[n],len-n,"\y3.\w Create Warning sign (East/West)^n")
			n += format( menu_body[n],len-n,"\y4.\w Delete Files^n")
			
			n += format( menu_body[n],len-n,"^n\y9. Back")
			n += format( menu_body[n],len-n,"^n\y0. Exit")
			
			keys = (1<<0)|(1<<1)|(1<<2)|(1<<3)|(1<<8)|(1<<9)
		}
	}
	show_menu(id,keys,menu_body,-1,"[Land Mines menu]")
}
//--------------------------------------------------------------------------------------------------
public action_menu(id, key)
{
	key++
	
	switch(MenuID[id]) {
		case 1: {
			switch(key) {
				case 1: {
					switch(get_pcvar_num(mines_enabled)) {
						case 0: {
							set_pcvar_num(mines_enabled, 1)
							client_print(0, print_chat, "[%s] Mines are now ON!", PLUGIN)
						}
						case 1: {
							set_pcvar_num(mines_enabled, 0)
							client_print(0, print_chat, "[%s] Mines are now OFF!", PLUGIN)
						}
					}
					show_minesmenu(id)
				}
				case 2: {
					switch(get_pcvar_num(mines_visible)) {
						case 0: {
							set_pcvar_num(mines_visible, 1)
							client_print(0, print_chat, "[%s] Mines will be visible after respawn!", PLUGIN)
						}
						case 1: {
							set_pcvar_num(mines_visible, 0)
							client_print(0, print_chat, "[%s] Mines will be invisible after respawn!", PLUGIN)
						}
					}
					show_minesmenu(id)
				}
				case 3: {
					MenuID[id] = 2
					show_minesmenu(id)
				}
				case 10: {
					MenuID[id] = 0
					return
				}
				default: show_minesmenu(id)
			}
		}
		case 2: {
			switch(key) {
				case 1: {
					new Float:origin[3]
					pev(id, pev_origin, origin)
					new writedata[128]
					formatex(writedata, 127, "%d %d %d", floatround(origin[0]), floatround(origin[1]), floatround(origin[2]))
					new configsdir[128]
					get_configsdir(configsdir, 127)
					
					new file[256], mapname[51]
					get_mapname(mapname, 50)
					format(file, 255, "%s/land_mines/%s_mines.cfg", configsdir, mapname)
					
					write_file(file, writedata)
					
					client_print(id, print_center, "[%s] Mine created at: X(%d) Y(%d) Z(%d)", PLUGIN, floatround(origin[0]), floatround(origin[1]), floatround(origin[2]))
					show_minesmenu(id)
				}
				case 2: {
					new Float:origin[3]
					pev(id, pev_origin, origin)
					new writedata[128]
					formatex(writedata, 127, "%d %d %d 0", floatround(origin[0]), floatround(origin[1]), floatround(origin[2]))
					new configsdir[128]
					get_configsdir(configsdir, 127)
					
					new file[256], mapname[51]
					get_mapname(mapname, 50)
					format(file, 255, "%s/land_mines/%s_signs.cfg", configsdir, mapname)
					
					write_file(file, writedata)
					
					client_print(id, print_center, "[%s] Sign (N/S) created at: X(%d) Y(%d) Z(%d)", PLUGIN, floatround(origin[0]), floatround(origin[1]), floatround(origin[2]))
					show_minesmenu(id)
				}
				case 3: {
					new Float:origin[3]
					pev(id, pev_origin, origin)
					new writedata[128]
					formatex(writedata, 127, "%d %d %d 1", floatround(origin[0]), floatround(origin[1]), floatround(origin[2]))
					new configsdir[128]
					get_configsdir(configsdir, 127)
					
					new file[256], mapname[51]
					get_mapname(mapname, 50)
					format(file, 255, "%s/land_mines/%s_signs.cfg", configsdir, mapname)
					
					write_file(file, writedata)
					
					client_print(id, print_center, "[%s] Sign (E/W) created at: X(%d) Y(%d) Z(%d)", PLUGIN, floatround(origin[0]), floatround(origin[1]), floatround(origin[2]))
					show_minesmenu(id)
				}
				case 4: {
					new configsdir[128]
					get_configsdir(configsdir, 127)
					
					new file[256], mapname[51]
					get_mapname(mapname, 50)
					format(file, 255, "%s/land_mines/%s_mines.cfg", configsdir, mapname)
					if(file_exists(file))
						delete_file(file)
					format(file, 255, "%s/land_mines/%s_signs.cfg", configsdir, mapname)
					if(file_exists(file))
						delete_file(file)
					
					show_minesmenu(id)
				}
				case 9: {
					MenuID[id] = 1
					show_minesmenu(id)
				}
				case 10: {
					MenuID[id] = 0
					return
				}
				default: show_minesmenu(id)
			}
		}
	}
}
//--------------------------------------------------------------------------------------------------
public mine_spawn(x[], y[], z[])
{
	if(!get_pcvar_num(mines_enabled))
		return
	
	new Float:origin[3]
	origin[0] = str_to_float(x)
	origin[1] = str_to_float(y)
	origin[2] = str_to_float(z)
	new ent = engfunc(EngFunc_CreateNamedEntity, engfunc(EngFunc_AllocString, "info_target"))
	set_pev(ent, pev_classname, "land_mine")
	set_pev(ent, pev_solid, 2)
	engfunc(EngFunc_SetModel, ent, "models/w_adrenaline.mdl")
	engfunc(EngFunc_SetSize, ent, Float:{ -10.0, -10.0, 0.0 }, Float:{ 10.0, 10.0, 4.0 } )
	engfunc(EngFunc_SetOrigin, ent, origin)
	engfunc(EngFunc_DropToFloor, ent)
	set_entity_visibility(ent, get_pcvar_num(mines_visible))
}
//--------------------------------------------------------------------------------------------------
public sign_spawn_ns(x[], y[], z[])
{
	if(!get_pcvar_num(mines_enabled))
		return
	
	new Float:origin[3]
	origin[0] = str_to_float(x)
	origin[1] = str_to_float(y)
	origin[2] = str_to_float(z)
	new ent = engfunc(EngFunc_CreateNamedEntity, engfunc(EngFunc_AllocString, "info_target"))
	set_pev(ent, pev_classname, "warning_sign")
	set_pev(ent, pev_solid, 2)
	engfunc(EngFunc_SetModel, ent, "models/land_mines/warning_ns.mdl")
	engfunc(EngFunc_SetSize, ent, Float:{ -1.1, -12.7, 0.0 }, Float:{ 1.1, 12.7, 38.0 } )
	engfunc(EngFunc_SetOrigin, ent, origin)
	engfunc(EngFunc_DropToFloor, ent)
	set_entity_visibility(ent, 1)
}
//--------------------------------------------------------------------------------------------------
public sign_spawn_ew(x[], y[], z[])
{
	if(!get_pcvar_num(mines_enabled))
		return
	
	new Float:origin[3]
	origin[0] = str_to_float(x)
	origin[1] = str_to_float(y)
	origin[2] = str_to_float(z)
	new ent = engfunc(EngFunc_CreateNamedEntity, engfunc(EngFunc_AllocString, "info_target"))
	set_pev(ent, pev_classname, "warning_sign")
	set_pev(ent, pev_solid, 2)
	engfunc(EngFunc_SetModel, ent, "models/land_mines/warning_ew.mdl")
	engfunc(EngFunc_SetSize, ent, Float:{ -12.7, -1.1, 0.0 }, Float:{ 12.7, 1.1, 38.0 } )
	engfunc(EngFunc_SetOrigin, ent, origin)
	engfunc(EngFunc_DropToFloor, ent)
	set_entity_visibility(ent, 1)
}
//--------------------------------------------------------------------------------------------------
public mine_touched(id, ent)
{
	if(!get_pcvar_num(mines_enabled))
		return
	
	if(!is_user_alive(id))
		return
	
	explode(ent)
}
//--------------------------------------------------------------------------------------------------
public explode(id)
{
	if(!get_pcvar_num(mines_enabled))
		return
	
	new Float:max_damage = get_pcvar_float(mines_maxdamage)
	new Float:radius = get_pcvar_float(mines_radius)
	new Float:distance
	new Float:damage
	new Float:origin[3]
	new Float:player_origin[3]
	pev(id, pev_origin, origin)
	
	for(new i = 0; i < 33; ++i) {
		if(!is_user_alive(i))
			continue
		
		pev(i, pev_origin, player_origin)
		distance = get_distance_f(origin, player_origin)
		if (distance < radius) {
			damage = max_damage - ((max_damage / radius) * distance)
			if(damage > 0) {
				if(get_user_health(i) <= damage) {
					user_silentkill(i)
					make_deathmsg(0, i, 0, "land_mine")
				}
				else
					fakedamage(i, "land_mine", damage, DMG_BLAST)
			}
		}
	}
	
	CreateExplosion(id)
	set_pev(id, pev_classname, "land_mine_dead")
	set_pev(id, pev_solid, 0)
	set_entity_visibility(id, 0)
	if(get_pcvar_num(mines_respawntime) > 0)
		set_task(get_pcvar_float(mines_respawntime), "mine_respawn", id)
}
//--------------------------------------------------------------------------------------------------
CreateExplosion(iCurrent)
{
	if(!get_pcvar_num(mines_enabled))
		return
	
	new Float:vOrigin[3];
	pev(iCurrent,pev_origin,vOrigin);

	message_begin(MSG_BROADCAST, SVC_TEMPENTITY);
	write_byte(99); //99 = KillBeam
	write_short(iCurrent);
	message_end();

	engfunc(EngFunc_MessageBegin, MSG_PVS, SVC_TEMPENTITY, vOrigin, 0);
	write_byte(TE_EXPLOSION);
	engfunc(EngFunc_WriteCoord,vOrigin[0]);
	engfunc(EngFunc_WriteCoord,vOrigin[1]);
	engfunc(EngFunc_WriteCoord,vOrigin[2]);
	write_short(boom);
	write_byte(get_pcvar_num(mines_radius));
	write_byte(15);
	write_byte(0);
	message_end();
}
//--------------------------------------------------------------------------------------------------
public mine_respawn(id)
{
	set_pev(id, pev_classname, "land_mine")
	set_pev(id, pev_solid, 2)
	set_entity_visibility(id, get_pcvar_num(mines_visible))
}
//--------------------------------------------------------------------------------------------------
/* AMXX-Studio Notes - DO NOT MODIFY BELOW HERE
*{\\ rtf1\\ fbidis\\ ansi\\ ansicpg1252\\ deff0{\\ fonttbl{\\ f0\\ fnil\\ fcharset0 Tahoma;}}\n\\ viewkind4\\ uc1\\ pard\\ ltrpar\\ lang1025\\ f0\\ fs16 \n\\ par }
*/
