/****************************************************\
|*Land Mines                                        *|
|*Created By: Sam Tsuki(Aka Sam X HearTLesS)        *|
|*                                                  *|
|* Have Fun,                                        *|
|*           Sam Tsuki                              *|
\****************************************************/

/*
Description:
This plugin creates Land Mines and Warning Signs by making map cfg files.

CMDs:
say /minesmenu		// ADMIN_KICK, Shows Land Mines menu

CVARs:
mines_enabled 1		// Enables Land Mines explosion
mines_visible 1		// Enables Land Mines t obe visible when spawned
mines_radius 200	// Maximum radius for Mines damage
mines_maxdamage 50	// Maximum damage a Mine can deal
mines_respawntime 20	// Seconds to respawn a Mine after explosion (Put to 0 to never respawn)
*/

#include <amxmodx>
#include <amxmisc>
#include <engine>
#include <fakemeta>
#include <hamsandwich>

#define PLUGIN "Land Mines"
#define VERSION "1.1"
#define AUTHOR "Sam Tsuki"

new mines_enabled, mines_visible, mines_radius, mines_maxdamage, mines_respawntime
new boom, MenuID[33]
new blood_drop, blood_spray
new mdl_gib_flesh, mdl_gib_head, mdl_gib_legbone, mdl_gib_lung, mdl_gib_meat, mdl_gib_spine
//--------------------------------------------------------------------------------------------------
public plugin_init()
{
	register_plugin(PLUGIN, VERSION, AUTHOR)
	
	mines_enabled = register_cvar("mines_enabled", "1")
	mines_visible = register_cvar("mines_visible", "1")
	mines_radius = register_cvar("mines_radius", "200")
	mines_maxdamage = register_cvar("mines_maxdamage", "50")
	mines_respawntime = register_cvar("mines_respawntime", "20")
	
	register_clcmd("say /minesmenu", "mines_menu", ADMIN_KICK, "- Shows Land Mines menu")
	
	register_clcmd("say_team /minesmenu", "mines_menu", ADMIN_KICK, "- Shows Land Mines menu")
	
	register_menucmd(register_menuid("[Land Mines menu]"),1023,"action_menu")
	
	register_touch("player", "land_mine", "mine_touched")
	
	register_event("ResetHUD","player_newspawn","b")
	
	set_task(0.1, "readminescfg")
	set_task(0.1, "readsignscfg")
	
	RegisterHam(Ham_TakeDamage, "info_target", "mines_damaged")
}
//--------------------------------------------------------------------------------------------------
public plugin_precache()
{
	precache_model("models/land_mines/land_mine.mdl")
	precache_model("models/land_mines/warning_ns.mdl")
	precache_model("models/land_mines/warning_ew.mdl")
	boom = precache_model("sprites/zerogxplode.spr")
	mdl_gib_flesh = precache_model("models/Fleshgibs.mdl")
	mdl_gib_head = precache_model("models/GIB_Skull.mdl")
	mdl_gib_legbone = precache_model("models/GIB_Legbone.mdl")
	mdl_gib_lung = precache_model("models/GIB_Lung.mdl")
	mdl_gib_meat = precache_model("models/GIB_B_Gib.mdl")
	mdl_gib_spine = precache_model("models/GIB_B_Bone.mdl")
}
//--------------------------------------------------------------------------------------------------
public readminescfg()
{
	new configsdir[128]
	get_configsdir(configsdir, 127)
	
	new file[256], mapname[51]
	get_mapname(mapname, 50)
	format(file, 255, "%s/land_mines/%s_mines.cfg", configsdir, mapname)
	
	if(file_exists(file)) {
		new readdata[128], txtlen
		new parsedx[8], parsedy[8], parsedz[8]
		new fsize = file_size(file, 1)
		for(new line = 0; line < fsize - 1; ++line) {
			read_file(file, line, readdata, 127, txtlen)
			parse(readdata, parsedx, 7, parsedy, 7, parsedz, 7)
			mine_spawn(parsedx, parsedy, parsedz)
		}
	}
}
//--------------------------------------------------------------------------------------------------
public readsignscfg()
{
	new configsdir[128]
	get_configsdir(configsdir, 127)
	
	new file[256], mapname[51]
	get_mapname(mapname, 50)
	format(file, 255, "%s/land_mines/%s_signs.cfg", configsdir, mapname)
	
	if(file_exists(file)) {
		new readdata[128], txtlen
		new parsedx[8], parsedy[8], parsedz[8], parseddir[8]
		new fsize = file_size(file, 1)
		for(new line = 0; line < fsize - 1; ++line) {
			read_file(file, line, readdata, 127, txtlen)
			parse(readdata, parsedx, 7, parsedy, 7, parsedz, 7, parseddir, 7)
			switch(str_to_num(parseddir)) {
				case 0: sign_spawn_ns(parsedx, parsedy, parsedz)
				case 1: sign_spawn_ew(parsedx, parsedy, parsedz)
			}
		}
	}
}
//--------------------------------------------------------------------------------------------------
public player_newspawn(id)
{
	fx_trans(id, 1)
}
//--------------------------------------------------------------------------------------------------
public mines_menu(id, level, cid)
{
	if(!cmd_access(id, level, cid, 0))
		return PLUGIN_HANDLED
	
	MenuID[id] = 1
	show_minesmenu(id)
	
	return PLUGIN_HANDLED
}
//--------------------------------------------------------------------------------------------------
public show_minesmenu(id)
{
	new menu_body[320]
	new n = 0
	new len = 319
	new keys
	
	switch(MenuID[id]) {
		case 1: {
			n += format( menu_body[n],len-n,"\yLand Mines menu:\w^n^n")
			
			switch(get_pcvar_num(mines_enabled)) {
				case 0: n += format( menu_body[n],len-n,"\y1.\w Turn Land Mines ON^n")
				case 1: n += format( menu_body[n],len-n,"\y1.\w Turn Land Mines OFF^n")
			}
			switch(get_pcvar_num(mines_visible)) {
				case 0: n += format( menu_body[n],len-n,"\y2.\w Turn Mines visiblity ON^n")
				case 1: n += format( menu_body[n],len-n,"\y2.\w Turn Mines visiblity OFF^n")
			}
			n += format( menu_body[n],len-n,"\y3.\w Edit Mode^n")
			
			n += format( menu_body[n],len-n,"^n\y0. Exit")
			
			keys = (1<<0)|(1<<1)|(1<<2)|(1<<9)
		}
		case 2: {
			n += format( menu_body[n],len-n,"\yLand Mines edit menu:\w^n^n")
			
			n += format( menu_body[n],len-n,"\y1.\w Create Mine^n")
			n += format( menu_body[n],len-n,"\y2.\w Create Warning sign (North/South)^n")
			n += format( menu_body[n],len-n,"\y3.\w Create Warning sign (East/West)^n")
			n += format( menu_body[n],len-n,"\y4.\w Delete Files^n")
			
			n += format( menu_body[n],len-n,"^n\y9. Back")
			n += format( menu_body[n],len-n,"^n\y0. Exit")
			
			keys = (1<<0)|(1<<1)|(1<<2)|(1<<3)|(1<<8)|(1<<9)
		}
	}
	show_menu(id,keys,menu_body,-1,"[Land Mines menu]")
}
//--------------------------------------------------------------------------------------------------
public action_menu(id, key)
{
	key++
	
	switch(MenuID[id]) {
		case 1: {
			switch(key) {
				case 1: {
					switch(get_pcvar_num(mines_enabled)) {
						case 0: {
							set_pcvar_num(mines_enabled, 1)
							client_print(0, print_chat, "[%s] Mines are now ON!", PLUGIN)
						}
						case 1: {
							set_pcvar_num(mines_enabled, 0)
							client_print(0, print_chat, "[%s] Mines are now OFF!", PLUGIN)
						}
					}
					show_minesmenu(id)
				}
				case 2: {
					switch(get_pcvar_num(mines_visible)) {
						case 0: {
							set_pcvar_num(mines_visible, 1)
							client_print(0, print_chat, "[%s] Mines will be visible after respawn!", PLUGIN)
						}
						case 1: {
							set_pcvar_num(mines_visible, 0)
							client_print(0, print_chat, "[%s] Mines will be invisible after respawn!", PLUGIN)
						}
					}
					show_minesmenu(id)
				}
				case 3: {
					MenuID[id] = 2
					show_minesmenu(id)
				}
				case 10: {
					MenuID[id] = 0
					return
				}
				default: show_minesmenu(id)
			}
		}
		case 2: {
			switch(key) {
				case 1: {
					new Float:origin[3]
					pev(id, pev_origin, origin)
					new writedata[128]
					formatex(writedata, 127, "%d %d %d", floatround(origin[0]), floatround(origin[1]), floatround(origin[2]))
					new configsdir[128]
					get_configsdir(configsdir, 127)
					
					new file[256], mapname[51]
					get_mapname(mapname, 50)
					format(file, 255, "%s/land_mines/%s_mines.cfg", configsdir, mapname)
					
					write_file(file, writedata)
					
					client_print(id, print_center, "[%s] Mine created at: X(%d) Y(%d) Z(%d)", PLUGIN, floatround(origin[0]), floatround(origin[1]), floatround(origin[2]))
					show_minesmenu(id)
				}
				case 2: {
					new Float:origin[3]
					pev(id, pev_origin, origin)
					new writedata[128]
					formatex(writedata, 127, "%d %d %d 0", floatround(origin[0]), floatround(origin[1]), floatround(origin[2]))
					new configsdir[128]
					get_configsdir(configsdir, 127)
					
					new file[256], mapname[51]
					get_mapname(mapname, 50)
					format(file, 255, "%s/land_mines/%s_signs.cfg", configsdir, mapname)
					
					write_file(file, writedata)
					
					client_print(id, print_center, "[%s] Sign (N/S) created at: X(%d) Y(%d) Z(%d)", PLUGIN, floatround(origin[0]), floatround(origin[1]), floatround(origin[2]))
					show_minesmenu(id)
				}
				case 3: {
					new Float:origin[3]
					pev(id, pev_origin, origin)
					new writedata[128]
					formatex(writedata, 127, "%d %d %d 1", floatround(origin[0]), floatround(origin[1]), floatround(origin[2]))
					new configsdir[128]
					get_configsdir(configsdir, 127)
					
					new file[256], mapname[51]
					get_mapname(mapname, 50)
					format(file, 255, "%s/land_mines/%s_signs.cfg", configsdir, mapname)
					
					write_file(file, writedata)
					
					client_print(id, print_center, "[%s] Sign (E/W) created at: X(%d) Y(%d) Z(%d)", PLUGIN, floatround(origin[0]), floatround(origin[1]), floatround(origin[2]))
					show_minesmenu(id)
				}
				case 4: {
					new configsdir[128]
					get_configsdir(configsdir, 127)
					
					new file[256], mapname[51]
					get_mapname(mapname, 50)
					format(file, 255, "%s/land_mines/%s_mines.cfg", configsdir, mapname)
					if(file_exists(file))
						delete_file(file)
					format(file, 255, "%s/land_mines/%s_signs.cfg", configsdir, mapname)
					if(file_exists(file))
						delete_file(file)
					
					show_minesmenu(id)
				}
				case 9: {
					MenuID[id] = 1
					show_minesmenu(id)
				}
				case 10: {
					MenuID[id] = 0
					return
				}
				default: show_minesmenu(id)
			}
		}
	}
}
//--------------------------------------------------------------------------------------------------
public mine_spawn(x[], y[], z[])
{
	if(!get_pcvar_num(mines_enabled))
		return
	
	new Float:origin[3]
	origin[0] = str_to_float(x)
	origin[1] = str_to_float(y)
	origin[2] = str_to_float(z)
	new ent = engfunc(EngFunc_CreateNamedEntity, engfunc(EngFunc_AllocString, "info_target"))
	set_pev(ent, pev_classname, "land_mine")
	set_pev(ent, pev_solid, 2)
	set_pev(ent, pev_takedamage, 1.0)
	set_pev(ent, pev_health, 9999.0)
	engfunc(EngFunc_SetModel, ent, "models/land_mines/land_mine.mdl")
	engfunc(EngFunc_SetSize, ent, Float:{ -4.0, -4.0, 0.0 }, Float:{ 4.0, 4.0, 4.0 } )
	engfunc(EngFunc_SetOrigin, ent, origin)
	engfunc(EngFunc_DropToFloor, ent)
	set_entity_visibility(ent, get_pcvar_num(mines_visible))
}
//--------------------------------------------------------------------------------------------------
public sign_spawn_ns(x[], y[], z[])
{
	if(!get_pcvar_num(mines_enabled))
		return
	
	new Float:origin[3]
	origin[0] = str_to_float(x)
	origin[1] = str_to_float(y)
	origin[2] = str_to_float(z)
	new ent = engfunc(EngFunc_CreateNamedEntity, engfunc(EngFunc_AllocString, "info_target"))
	set_pev(ent, pev_classname, "warning_sign")
	set_pev(ent, pev_solid, 2)
	engfunc(EngFunc_SetModel, ent, "models/land_mines/warning_ns.mdl")
	engfunc(EngFunc_SetSize, ent, Float:{ -1.1, -12.7, 0.0 }, Float:{ 1.1, 12.7, 38.0 } )
	engfunc(EngFunc_SetOrigin, ent, origin)
	engfunc(EngFunc_DropToFloor, ent)
	set_entity_visibility(ent, 1)
}
//--------------------------------------------------------------------------------------------------
public sign_spawn_ew(x[], y[], z[])
{
	if(!get_pcvar_num(mines_enabled))
		return
	
	new Float:origin[3]
	origin[0] = str_to_float(x)
	origin[1] = str_to_float(y)
	origin[2] = str_to_float(z)
	new ent = engfunc(EngFunc_CreateNamedEntity, engfunc(EngFunc_AllocString, "info_target"))
	set_pev(ent, pev_classname, "warning_sign")
	set_pev(ent, pev_solid, 2)
	engfunc(EngFunc_SetModel, ent, "models/land_mines/warning_ew.mdl")
	engfunc(EngFunc_SetSize, ent, Float:{ -12.7, -1.1, 0.0 }, Float:{ 12.7, 1.1, 38.0 } )
	engfunc(EngFunc_SetOrigin, ent, origin)
	engfunc(EngFunc_DropToFloor, ent)
	set_entity_visibility(ent, 1)
}
//--------------------------------------------------------------------------------------------------
public mine_touched(id, ent)
{
	if(!get_pcvar_num(mines_enabled))
		return
	
	if(!is_user_alive(id))
		return
	
	explode(ent)
}
//--------------------------------------------------------------------------------------------------
public mines_damaged(this, idinflictor, idattacker, Float:damage, damagebits)
{
	new classname[32]
	pev(this, pev_classname, classname, 31)
	if(!equal(classname, "land_mine"))
		return HAM_IGNORED
	
	explode(this)
	return HAM_HANDLED
}
//--------------------------------------------------------------------------------------------------
public explode(id)
{
	if(!get_pcvar_num(mines_enabled))
		return
	
	new Float:max_damage = get_pcvar_float(mines_maxdamage)
	new Float:radius = get_pcvar_float(mines_radius)
	new Float:distance
	new Float:damage
	new Float:origin[3]
	new Float:player_origin[3]
	pev(id, pev_origin, origin)
	
	for(new i = 0; i < 33; ++i) {
		if(!is_user_alive(i))
			continue
		
		pev(i, pev_origin, player_origin)
		distance = get_distance_f(origin, player_origin)
		if (distance < radius) {
			damage = max_damage - ((max_damage / radius) * distance)
			if(damage > 0) {
				if(get_user_health(i) <= damage) {
					// Effects
					new iOrigin[3]
					get_user_origin(i, iOrigin)
					// Effects
					fx_blood_red(iOrigin)
					fx_blood_red(iOrigin)
					fx_blood_red(iOrigin)
					fx_bleed_red(iOrigin)
					fx_bleed_red(iOrigin)
					fx_headshot_red(iOrigin)
					fx_trans(i,0)
					fx_gib_explode(iOrigin)
					fx_blood_large_red(iOrigin,10)
					fx_blood_small_red(iOrigin,25)
					// Hide body
					iOrigin[2] = iOrigin[2]-20
					set_pev(i, pev_origin, player_origin)
					user_silentkill(i)
					make_deathmsg(0, i, 0, "land_mine")
				}
				else
					fakedamage(i, "land_mine", damage, DMG_BLAST)
			}
		}
	}
	
	CreateExplosion(id)
	set_pev(id, pev_classname, "land_mine_dead")
	set_pev(id, pev_solid, 0)
	set_pev(id, pev_takedamage, 0.0)
	set_pev(id, pev_health, 9999.0)
	set_entity_visibility(id, 0)
	if(get_pcvar_num(mines_respawntime) > 0)
		set_task(get_pcvar_float(mines_respawntime), "mine_respawn", id)
}
//--------------------------------------------------------------------------------------------------
CreateExplosion(iCurrent)
{
	if(!get_pcvar_num(mines_enabled))
		return
	
	new Float:vOrigin[3];
	pev(iCurrent,pev_origin,vOrigin);

	message_begin(MSG_BROADCAST, SVC_TEMPENTITY);
	write_byte(99); //99 = KillBeam
	write_short(iCurrent);
	message_end();

	engfunc(EngFunc_MessageBegin, MSG_PVS, SVC_TEMPENTITY, vOrigin, 0);
	write_byte(TE_EXPLOSION);
	engfunc(EngFunc_WriteCoord,vOrigin[0]);
	engfunc(EngFunc_WriteCoord,vOrigin[1]);
	engfunc(EngFunc_WriteCoord,vOrigin[2] + 30);
	write_short(boom);
	write_byte(30);
	write_byte(15);
	write_byte(0);
	message_end();
	
	static const explosion[3] = {46,47,48}
	
	message_begin(MSG_BROADCAST,SVC_TEMPENTITY)
	write_byte(116)
	engfunc(EngFunc_WriteCoord,vOrigin[0])
	engfunc(EngFunc_WriteCoord,vOrigin[1])
	engfunc(EngFunc_WriteCoord,vOrigin[2])
	write_byte(explosion[random_num(0,2)])
	message_end()
}
//--------------------------------------------------------------------------------------------------
public mine_respawn(id)
{
	set_pev(id, pev_classname, "land_mine")
	set_pev(id, pev_solid, 2)
	set_pev(id, pev_takedamage, 1.0)
	set_pev(id, pev_health, 9999.0)
	set_entity_visibility(id, get_pcvar_num(mines_visible))
}
//--------------------------------------------------------------------------------------------------
static fx_trans(player,amount)
{
	set_entity_visibility(player, amount)
}
//--------------------------------------------------------------------------------------------------
public fx_blood_red(origin[3]) //hit blood human
 {
	message_begin(MSG_BROADCAST,SVC_TEMPENTITY)
	write_byte(115) //TE_BLOODSPRITE in const.h
	write_coord(origin[0]+random_num(-20,20))
	write_coord(origin[1]+random_num(-20,20))
	write_coord(origin[2]+random_num(-20,20))
	write_short(blood_spray)
	write_short(blood_drop)
	write_byte(248) // color index
	write_byte(15) // size
	message_end()
 }
 //-------------------------------------------------------------------------------------------------
 public fx_bleed_red(origin[3]) //blood sprays on low hp
{
	// Blood spray
	message_begin(MSG_BROADCAST,SVC_TEMPENTITY)
	write_byte(101)
	write_coord(origin[0])
	write_coord(origin[1])
	write_coord(origin[2]+10)
	write_coord(random_num(-100,100)) // x
	write_coord(random_num(-100,100)) // y
	write_coord(random_num(-10,10)) // z
	write_byte(70) // color
	write_byte(random_num(50,100)) // speed
	message_end()
}
//--------------------------------------------------------------------------------------------------
public fx_headshot_red(origin[3])
{
	// Blood spray, 5 times
	for (new i = 0; i < 5; i++) {
		message_begin(MSG_BROADCAST,SVC_TEMPENTITY)
		write_byte(101)
		write_coord(origin[0])
		write_coord(origin[1])
		write_coord(origin[2]+30)
		write_coord(random_num(-20,20)) // x
		write_coord(random_num(-20,20)) // y
		write_coord(random_num(50,300)) // z
		write_byte(70) // color
		write_byte(random_num(100,200)) // speed
		message_end()
	}
}
//--------------------------------------------------------------------------------------------------
static fx_blood_small_red(origin[3],num) //red blood decals [small]
{
	// Blood decals
	static const blood_small[7] = {190,191,192,193,194,195,197}
	// Small splash
	for (new j = 0; j < num; j++) {
		message_begin(MSG_BROADCAST,SVC_TEMPENTITY)
		write_byte(116) //TE_WORLDDECAL from const.h
		write_coord(origin[0]+random_num(-100,100))
		write_coord(origin[1]+random_num(-100,100))
		write_coord(origin[2]-36)
		write_byte(blood_small[random_num(0,6)]) // index
		message_end()
	}
}
//--------------------------------------------------------------------------------------------------
static fx_blood_large_red(origin[3],num) //red blood decals [ big]
{
	// Blood decals
	static const blood_large[2] = {204,205}

	// Large splash
	for (new i = 0; i < num; i++) {
		message_begin(MSG_BROADCAST,SVC_TEMPENTITY)
		write_byte(116) //TE_WORLDDECAL from const.h
		write_coord(origin[0]+random_num(-50,50))
		write_coord(origin[1]+random_num(-50,50))
		write_coord(origin[2]-36)
		write_byte(blood_large[random_num(0,1)]) // index
		message_end()
	}
}
//--------------------------------------------------------------------------------------------------
static fx_gib_explode(origin[3])
{
	new flesh[3]
	flesh[0] = mdl_gib_flesh
	flesh[1] = mdl_gib_meat
	flesh[2] = mdl_gib_legbone

	// Gib explosion
	// Head
	message_begin(MSG_BROADCAST,SVC_TEMPENTITY)
	write_byte(106) //TR_MODEL
	write_coord(origin[0])
	write_coord(origin[1])
	write_coord(origin[2])
	write_coord(random_num(-100,100))
	write_coord(random_num(-100,100))
	write_coord(random_num(100,200))
	write_angle(random_num(0,360))
	write_short(mdl_gib_head)
	write_byte(0) // bounce
	write_byte(500) // life
	message_end()
	
	// Spine
	message_begin(MSG_BROADCAST,SVC_TEMPENTITY)
	write_byte(106)
	write_coord(origin[0])
	write_coord(origin[1])
	write_coord(origin[2])
	write_coord(random_num(-100,100))
	write_coord(random_num(-100,100))
	write_coord(random_num(100,200))
	write_angle(random_num(0,360))
	write_short(mdl_gib_spine)
	write_byte(0) // bounce
	write_byte(500) // life
	message_end()
	
	// Lung
	for(new i = 0; i < random_num(1,2); i++) {
		message_begin(MSG_BROADCAST,SVC_TEMPENTITY)
		write_byte(106)
		write_coord(origin[0])
		write_coord(origin[1])
		write_coord(origin[2])
		write_coord(random_num(-100,100))
		write_coord(random_num(-100,100))
		write_coord(random_num(100,200))
		write_angle(random_num(0,360))
		write_short(mdl_gib_lung)
		write_byte(0) // bounce
		write_byte(500) // life
		message_end()
	}
	
	// Parts, 5 times
	for(new i = 0; i < 5; i++) {
		message_begin(MSG_BROADCAST,SVC_TEMPENTITY)
		write_byte(106)
		write_coord(origin[0])
		write_coord(origin[1])
		write_coord(origin[2])
		write_coord(random_num(-100,100))
		write_coord(random_num(-100,100))
		write_coord(random_num(100,200))
		write_angle(random_num(0,360))
		write_short(flesh[random_num(0,2)])
		write_byte(0) // bounce
		write_byte(500) // life
		message_end()
	}
}
//--------------------------------------------------------------------------------------------------
/* AMXX-Studio Notes - DO NOT MODIFY BELOW HERE
*{\\ rtf1\\ fbidis\\ ansi\\ ansicpg1252\\ deff0{\\ fonttbl{\\ f0\\ fnil\\ fcharset0 Tahoma;}}\n\\ viewkind4\\ uc1\\ pard\\ ltrpar\\ lang1025\\ f0\\ fs16 \n\\ par }
*/
